@@ -157,9 +157,9 @@ gst_ass_render_class_init (GstAssRenderClass * klass)
 
 #if defined(LIBASS_VERSION) && LIBASS_VERSION >= 0x00907000
 static void
-_libass_message_cb (gint level, const gchar * fmt, va_list args, gpointer data)
+_libass_message_cb (gint level, const gchar * fmt, va_list args,
+    gpointer render)
 {
-  GstAssRender *render = GST_ASS_RENDER (data);
   gchar *message = g_strdup_vprintf (fmt, args);
 
   if (level < 2)
@@ -480,7 +480,7 @@ blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
   gint height = render->height; \
   gint dst_stride = GST_ROUND_UP_4 (width * bpp); \
   gint dst_skip; \
-  gint src_stride, src_skip; \
+  gint src_skip; \
   \
   while (ass_image) { \
     if (ass_image->dst_y > height || ass_image->dst_x > width) \
@@ -496,7 +496,6 @@ blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
     \
     w = MIN (ass_image->w, width - ass_image->dst_x); \
     h = MIN (ass_image->h, height - ass_image->dst_y); \
-    src_stride = ass_image->stride; \
     src_skip = ass_image->stride - w; \
     dst_skip = dst_stride - w * bpp; \
     \
@@ -571,13 +570,13 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
   const guint8 *src;
   guint8 *dst_y, *dst_u, *dst_v;
   gint x, y, w, h;
-  gint w2, h2;
+  gint w2;
   gint width = render->width;
   gint height = render->height;
   gint src_stride;
-  gint y_offset, y_height, y_width, y_stride;
-  gint u_offset, u_height, u_width, u_stride;
-  gint v_offset, v_height, v_width, v_stride;
+  gint y_offset, y_stride;
+  gint u_offset, u_stride;
+  gint v_offset, v_stride;
 
   y_offset =
       gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 0, width,
@@ -589,20 +588,6 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
       gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 2, width,
       height);
 
-  y_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 0, height);
-  u_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 1, height);
-  v_height =
-      gst_video_format_get_component_height (GST_VIDEO_FORMAT_I420, 2, height);
-
-  y_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 0, width);
-  u_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 1, width);
-  v_width =
-      gst_video_format_get_component_width (GST_VIDEO_FORMAT_I420, 2, width);
-
   y_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 0, width);
   u_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 1, width);
   v_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 2, width);
@@ -625,7 +610,6 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
     h = MIN (ass_image->h, height - ass_image->dst_y);
 
     w2 = (w + 1) / 2;
-    h2 = (h + 1) / 2;
 
     src_stride = ass_image->stride;
 
@@ -634,101 +618,130 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
         buffer->data + y_offset + ass_image->dst_y * y_stride +
         ass_image->dst_x;
     dst_u =
-        buffer->data + u_offset + ((ass_image->dst_y + 1) / 2) * u_stride +
-        (ass_image->dst_x + 1) / 2;
+        buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
+        ass_image->dst_x / 2;
     dst_v =
-        buffer->data + v_offset + ((ass_image->dst_y + 1) / 2) * v_stride +
-        (ass_image->dst_x + 1) / 2;
+        buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
+        ass_image->dst_x / 2;
 
-    for (y = 0; y < h - 1; y += 2) {
-      for (x = 0; x < w - 1; x += 2) {
-        k = src[0] * alpha / 255;
-        k2 = k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
+    for (y = 0; y < h; y++) {
+      dst_y = buffer->data + y_offset + (ass_image->dst_y + y) * y_stride +
+          ass_image->dst_x;
+      for (x = 0; x < w; x++) {
+        k = src[y * ass_image->w + x] * alpha / 255;
+        dst_y[x] = (k * Y + (255 - k) * dst_y[x]) / 255;
+      }
+    }
 
-        k = src[1] * alpha / 255;
-        k2 += k;
-        dst_y[1] = (k * Y + (255 - k) * dst_y[1]) / 255;
-
-        src += src_stride;
-        dst_y += y_stride;
-
-        k = src[0] * alpha / 255;
-        k2 += k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
-
-        k = src[1] * alpha / 255;
-        k2 += k;
-        dst_y[1] = (k * Y + (255 - k) * dst_y[1]) / 255;
-
-        k2 /= 4;
+    y = 0;
+    if (ass_image->dst_y & 1) {
+      dst_u =
+          buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
+          ass_image->dst_x / 2;
+      dst_v =
+          buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
+          ass_image->dst_x / 2;
+      x = 0;
+      if (ass_image->dst_x & 1) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
+        dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
+        dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
+        x++;
+        dst_u++;
+        dst_v++;
+      }
+      for (; x < w - 1; x += 2) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 += src[y * ass_image->w + x + 1] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
         dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
         dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
         dst_u++;
         dst_v++;
-
-        src += -src_stride + 2;
-        dst_y += -y_stride + 2;
       }
-
       if (x < w) {
-        k = src[0] * alpha / 255;
-        k2 = k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
+        dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
+        dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
+      }
+    }
 
-        src += src_stride;
-        dst_y += y_stride;
-
-        k = src[0] * alpha / 255;
-        k2 += k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
-
-        k2 /= 2;
+    for (; y < h - 1; y += 2) {
+      dst_u =
+          buffer->data + u_offset + ((ass_image->dst_y + y) / 2) * u_stride +
+          ass_image->dst_x / 2;
+      dst_v =
+          buffer->data + v_offset + ((ass_image->dst_y + y) / 2) * v_stride +
+          ass_image->dst_x / 2;
+      x = 0;
+      if (ass_image->dst_x & 1) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 += src[(y + 1) * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
+        dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
+        dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
+        x++;
+        dst_u++;
+        dst_v++;
+      }
+      for (; x < w - 1; x += 2) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 += src[y * ass_image->w + x + 1] * alpha / 255;
+        k2 += src[(y + 1) * ass_image->w + x] * alpha / 255;
+        k2 += src[(y + 1) * ass_image->w + x + 1] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
         dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
         dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
         dst_u++;
         dst_v++;
-
-        src += -src_stride + 1;
-        dst_y += -y_stride + 1;
       }
-
-      src += src_stride + (src_stride - w);
-      dst_y += y_stride + (y_stride - w);
-      dst_u += u_stride - w2;
-      dst_v += v_stride - w2;
+      if (x < w) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 += src[(y + 1) * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
+        dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
+        dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
+      }
     }
 
     if (y < h) {
-      for (x = 0; x < w - 1; x += 2) {
-        k = src[0] * alpha / 255;
-        k2 = k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
-
-        k = src[1] * alpha / 255;
-        k2 += k;
-        dst_y[1] = (k * Y + (255 - k) * dst_y[1]) / 255;
-
-        k2 /= 2;
+      dst_u =
+          buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
+          ass_image->dst_x / 2;
+      dst_v =
+          buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
+          ass_image->dst_x / 2;
+      x = 0;
+      if (ass_image->dst_x & 1) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
+        dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
+        dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
+        x++;
+        dst_u++;
+        dst_v++;
+      }
+      for (; x < w - 1; x += 2) {
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 += src[y * ass_image->w + x + 1] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
         dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
         dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
         dst_u++;
         dst_v++;
-
-        src += 2;
-        dst_y += 2;
       }
-
       if (x < w) {
-        k = src[0] * alpha / 255;
-        k2 = k;
-        dst_y[0] = (k * Y + (255 - k) * dst_y[0]) / 255;
-
+        k2 = src[y * ass_image->w + x] * alpha / 255;
+        k2 = (k2 + 2) >> 2;
         dst_u[0] = (k2 * U + (255 - k2) * dst_u[0]) / 255;
         dst_v[0] = (k2 * V + (255 - k2) * dst_v[0]) / 255;
       }
     }
 
+
+
   next:
     counter++;
     ass_image = ass_image->next;
