@@ -446,11 +446,12 @@ gst_rsvg_overlay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &video_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &data_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&data_sink_template));
 
   gst_element_class_set_details_simple (element_class, "RSVG overlay",
       "Filter/Editor/Video",
