@@ -65,30 +65,8 @@ typedef struct
   GList *invisible;
 } GstVP8EncCoderHook;
 
-static void
-_gst_mini_object_unref0 (GstMiniObject * obj)
-{
-  if (obj)
-    gst_mini_object_unref (obj);
-}
-
-static void
-gst_vp8_enc_coder_hook_free (GstVP8EncCoderHook * hook)
-{
-  if (hook->image)
-    g_slice_free (vpx_image_t, hook->image);
-
-  g_list_foreach (hook->invisible, (GFunc) _gst_mini_object_unref0, NULL);
-  g_list_free (hook->invisible);
-  g_slice_free (GstVP8EncCoderHook, hook);
-}
-
 #define DEFAULT_BITRATE 0
 #define DEFAULT_MODE VPX_VBR
-#define DEFAULT_MINSECTION_PCT 5
-#define DEFAULT_MAXSECTION_PCT 800
-#define DEFAULT_MIN_QUANTIZER 0
-#define DEFAULT_MAX_QUANTIZER 63
 #define DEFAULT_QUALITY 5
 #define DEFAULT_ERROR_RESILIENT FALSE
 #define DEFAULT_MAX_LATENCY 10
@@ -96,33 +74,14 @@ gst_vp8_enc_coder_hook_free (GstVP8EncCoderHook * hook)
 #define DEFAULT_SPEED 0
 #define DEFAULT_THREADS 1
 #define DEFAULT_MULTIPASS_MODE VPX_RC_ONE_PASS
-#define DEFAULT_MULTIPASS_CACHE_FILE "multipass.cache"
+#define DEFAULT_MULTIPASS_CACHE_FILE NULL
 #define DEFAULT_AUTO_ALT_REF_FRAMES FALSE
-#define DEFAULT_LAG_IN_FRAMES 0
-#define DEFAULT_SHARPNESS 0
-#define DEFAULT_NOISE_SENSITIVITY 0
-#ifdef HAVE_VP8ENC_TUNING
-#define DEFAULT_TUNE VP8_TUNE_PSNR
-#else
-typedef enum
-{ VP8_TUNE_NONE } vp8e_tuning;
-#define DEFAULT_TUNE VP8_TUNE_NONE
-#endif
-#define DEFAULT_STATIC_THRESHOLD 0
-#define DEFAULT_DROP_FRAME 0
-#define DEFAULT_RESIZE_ALLOWED TRUE
-#define DEFAULT_TOKEN_PARTS 0
-
 
 enum
 {
   PROP_0,
   PROP_BITRATE,
   PROP_MODE,
-  PROP_MINSECTION_PCT,
-  PROP_MAXSECTION_PCT,
-  PROP_MIN_QUANTIZER,
-  PROP_MAX_QUANTIZER,
   PROP_QUALITY,
   PROP_ERROR_RESILIENT,
   PROP_MAX_LATENCY,
@@ -131,15 +90,7 @@ enum
   PROP_THREADS,
   PROP_MULTIPASS_MODE,
   PROP_MULTIPASS_CACHE_FILE,
-  PROP_AUTO_ALT_REF_FRAMES,
-  PROP_LAG_IN_FRAMES,
-  PROP_SHARPNESS,
-  PROP_NOISE_SENSITIVITY,
-  PROP_TUNE,
-  PROP_STATIC_THRESHOLD,
-  PROP_DROP_FRAME,
-  PROP_RESIZE_ALLOWED,
-  PROP_TOKEN_PARTS
+  PROP_AUTO_ALT_REF_FRAMES
 };
 
 #define GST_VP8_ENC_MODE_TYPE (gst_vp8_enc_mode_get_type())
@@ -187,32 +138,6 @@ gst_vp8_enc_multipass_mode_get_type (void)
   return id;
 }
 
-#define GST_VP8_ENC_TUNE_TYPE (gst_vp8_enc_tune_get_type())
-static GType
-gst_vp8_enc_tune_get_type (void)
-{
-  static const GEnumValue values[] = {
-#ifdef HAVE_VP8ENC_TUNING
-    {VP8_TUNE_PSNR, "Tune for PSNR", "psnr"},
-    {VP8_TUNE_SSIM, "Tune for SSIM", "ssim"},
-#else
-    {VP8_TUNE_NONE, "none", "none"},
-#endif
-    {0, NULL, NULL}
-  };
-  static volatile GType id = 0;
-
-  if (g_once_init_enter ((gsize *) & id)) {
-    GType _id;
-
-    _id = g_enum_register_static ("GstVP8EncTune", values);
-
-    g_once_init_leave ((gsize *) & id, _id);
-  }
-
-  return id;
-}
-
 static void gst_vp8_enc_finalize (GObject * object);
 static void gst_vp8_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -224,12 +149,13 @@ static gboolean gst_vp8_enc_stop (GstBaseVideoEncoder * encoder);
 static gboolean gst_vp8_enc_set_format (GstBaseVideoEncoder *
     base_video_encoder, GstVideoState * state);
 static gboolean gst_vp8_enc_finish (GstBaseVideoEncoder * base_video_encoder);
-static GstFlowReturn gst_vp8_enc_handle_frame (GstBaseVideoEncoder *
+static gboolean gst_vp8_enc_handle_frame (GstBaseVideoEncoder *
     base_video_encoder, GstVideoFrame * frame);
 static GstFlowReturn gst_vp8_enc_shape_output (GstBaseVideoEncoder * encoder,
     GstVideoFrame * frame);
-static gboolean gst_vp8_enc_sink_event (GstBaseVideoEncoder *
-    base_video_encoder, GstEvent * event);
+static GstCaps *gst_vp8_enc_get_caps (GstBaseVideoEncoder * base_video_encoder);
+
+static gboolean gst_vp8_enc_sink_event (GstPad * pad, GstEvent * event);
 
 static GstStaticPadTemplate gst_vp8_enc_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -269,10 +195,10 @@ gst_vp8_enc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_vp8_enc_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_vp8_enc_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_vp8_enc_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_vp8_enc_sink_template));
 
   gst_element_class_set_details_simple (element_class,
       "On2 VP8 Encoder",
@@ -299,7 +225,7 @@ gst_vp8_enc_class_init (GstVP8EncClass * klass)
   base_video_encoder_class->set_format = gst_vp8_enc_set_format;
   base_video_encoder_class->finish = gst_vp8_enc_finish;
   base_video_encoder_class->shape_output = gst_vp8_enc_shape_output;
-  base_video_encoder_class->event = gst_vp8_enc_sink_event;
+  base_video_encoder_class->get_caps = gst_vp8_enc_get_caps;
 
   g_object_class_install_property (gobject_class, PROP_BITRATE,
       g_param_spec_int ("bitrate", "Bit rate",
@@ -313,35 +239,9 @@ gst_vp8_enc_class_init (GstVP8EncClass * klass)
           GST_VP8_ENC_MODE_TYPE, DEFAULT_MODE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
-  g_object_class_install_property (gobject_class, PROP_MINSECTION_PCT,
-      g_param_spec_uint ("minsection-pct",
-          "minimum percentage allocation per section",
-          "The numbers represent a percentage of the average allocation per section (frame)",
-          0, 20, DEFAULT_MINSECTION_PCT,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_MAXSECTION_PCT,
-      g_param_spec_uint ("maxsection-pct",
-          "maximum percentage allocation per section",
-          "The numbers represent a percentage of the average allocation per section (frame)",
-          200, 800, DEFAULT_MAXSECTION_PCT,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_MIN_QUANTIZER,
-      g_param_spec_int ("min-quantizer", "Minimum quantizer",
-          "Minimum (best) quantizer",
-          0, 63, DEFAULT_MIN_QUANTIZER,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_MAX_QUANTIZER,
-      g_param_spec_int ("max-quantizer", "Maximum quantizer",
-          "Maximum (worst) quantizer",
-          0, 63, DEFAULT_MAX_QUANTIZER,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
   g_object_class_install_property (gobject_class, PROP_QUALITY,
       g_param_spec_double ("quality", "Quality",
-          "Quality. This parameter sets a constant quantizer.",
+          "Quality",
           0, 10.0, DEFAULT_QUALITY,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
@@ -366,7 +266,7 @@ gst_vp8_enc_class_init (GstVP8EncClass * klass)
   g_object_class_install_property (gobject_class, PROP_SPEED,
       g_param_spec_int ("speed", "Speed",
           "Speed",
-          0, 7, DEFAULT_SPEED,
+          0, 2, DEFAULT_SPEED,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (gobject_class, PROP_THREADS,
@@ -393,54 +293,6 @@ gst_vp8_enc_class_init (GstVP8EncClass * klass)
           DEFAULT_AUTO_ALT_REF_FRAMES,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
-  g_object_class_install_property (gobject_class, PROP_LAG_IN_FRAMES,
-      g_param_spec_uint ("lag-in-frames", "Max number of frames to lag",
-          "If set, this value allows the encoder to consume a number of input "
-          "frames before producing output frames.",
-          0, 64, DEFAULT_LAG_IN_FRAMES,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_SHARPNESS,
-      g_param_spec_int ("sharpness", "Sharpness",
-          "Sharpness",
-          0, 7, DEFAULT_SHARPNESS,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_NOISE_SENSITIVITY,
-      g_param_spec_int ("noise-sensitivity", "Noise Sensitivity",
-          "Noise Sensitivity",
-          0, 6, DEFAULT_NOISE_SENSITIVITY,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_TUNE,
-      g_param_spec_enum ("tune", "Tune",
-          "Tune",
-          GST_VP8_ENC_TUNE_TYPE, DEFAULT_TUNE,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_STATIC_THRESHOLD,
-      g_param_spec_int ("static-threshold", "Static Threshold",
-          "Static Threshold",
-          0, 1000, DEFAULT_STATIC_THRESHOLD,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_DROP_FRAME,
-      g_param_spec_int ("drop-frame", "Drop Frame",
-          "Drop Frame",
-          0, 100, DEFAULT_DROP_FRAME,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_RESIZE_ALLOWED,
-      g_param_spec_boolean ("resize-allowed", "Resize Allowed",
-          "Resize Allowed",
-          DEFAULT_RESIZE_ALLOWED,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
-  g_object_class_install_property (gobject_class, PROP_TOKEN_PARTS,
-      g_param_spec_int ("token-parts", "Token Parts",
-          "Token Parts",
-          0, 3, DEFAULT_TOKEN_PARTS,
-          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   GST_DEBUG_CATEGORY_INIT (gst_vp8enc_debug, "vp8enc", 0, "VP8 Encoder");
 }
@@ -452,19 +304,20 @@ gst_vp8_enc_init (GstVP8Enc * gst_vp8_enc, GstVP8EncClass * klass)
   GST_DEBUG_OBJECT (gst_vp8_enc, "init");
 
   gst_vp8_enc->bitrate = DEFAULT_BITRATE;
-  gst_vp8_enc->minsection_pct = DEFAULT_MINSECTION_PCT;
-  gst_vp8_enc->maxsection_pct = DEFAULT_MAXSECTION_PCT;
-  gst_vp8_enc->min_quantizer = DEFAULT_MIN_QUANTIZER;
-  gst_vp8_enc->max_quantizer = DEFAULT_MAX_QUANTIZER;
   gst_vp8_enc->mode = DEFAULT_MODE;
   gst_vp8_enc->quality = DEFAULT_QUALITY;
   gst_vp8_enc->error_resilient = DEFAULT_ERROR_RESILIENT;
   gst_vp8_enc->max_latency = DEFAULT_MAX_LATENCY;
   gst_vp8_enc->max_keyframe_distance = DEFAULT_MAX_KEYFRAME_DISTANCE;
   gst_vp8_enc->multipass_mode = DEFAULT_MULTIPASS_MODE;
-  gst_vp8_enc->multipass_cache_file = g_strdup (DEFAULT_MULTIPASS_CACHE_FILE);
+  gst_vp8_enc->multipass_cache_file = DEFAULT_MULTIPASS_CACHE_FILE;
   gst_vp8_enc->auto_alt_ref_frames = DEFAULT_AUTO_ALT_REF_FRAMES;
-  gst_vp8_enc->lag_in_frames = DEFAULT_LAG_IN_FRAMES;
+
+  /* FIXME: Add sink/src event vmethods */
+  gst_vp8_enc->base_sink_event_func =
+      GST_PAD_EVENTFUNC (GST_BASE_VIDEO_CODEC_SINK_PAD (gst_vp8_enc));
+  gst_pad_set_event_function (GST_BASE_VIDEO_CODEC_SINK_PAD (gst_vp8_enc),
+      gst_vp8_enc_sink_event);
 }
 
 static void
@@ -501,18 +354,6 @@ gst_vp8_enc_set_property (GObject * object, guint prop_id,
     case PROP_MODE:
       gst_vp8_enc->mode = g_value_get_enum (value);
       break;
-    case PROP_MINSECTION_PCT:
-      gst_vp8_enc->minsection_pct = g_value_get_uint (value);
-      break;
-    case PROP_MAXSECTION_PCT:
-      gst_vp8_enc->maxsection_pct = g_value_get_uint (value);
-      break;
-    case PROP_MIN_QUANTIZER:
-      gst_vp8_enc->min_quantizer = g_value_get_int (value);
-      break;
-    case PROP_MAX_QUANTIZER:
-      gst_vp8_enc->max_quantizer = g_value_get_int (value);
-      break;
     case PROP_QUALITY:
       gst_vp8_enc->quality = g_value_get_double (value);
       break;
@@ -542,35 +383,6 @@ gst_vp8_enc_set_property (GObject * object, guint prop_id,
     case PROP_AUTO_ALT_REF_FRAMES:
       gst_vp8_enc->auto_alt_ref_frames = g_value_get_boolean (value);
       break;
-    case PROP_LAG_IN_FRAMES:
-      gst_vp8_enc->lag_in_frames = g_value_get_uint (value);
-      break;
-    case PROP_SHARPNESS:
-      gst_vp8_enc->sharpness = g_value_get_int (value);
-      break;
-    case PROP_NOISE_SENSITIVITY:
-      gst_vp8_enc->noise_sensitivity = g_value_get_int (value);
-      break;
-    case PROP_TUNE:
-#ifdef HAVE_VP8ENC_TUNING
-      gst_vp8_enc->tuning = g_value_get_enum (value);
-#else
-      GST_WARNING_OBJECT (gst_vp8_enc,
-          "The tuning property is unsupported by this libvpx");
-#endif
-      break;
-    case PROP_STATIC_THRESHOLD:
-      gst_vp8_enc->static_threshold = g_value_get_int (value);
-      break;
-    case PROP_DROP_FRAME:
-      gst_vp8_enc->drop_frame = g_value_get_int (value);
-      break;
-    case PROP_RESIZE_ALLOWED:
-      gst_vp8_enc->resize_allowed = g_value_get_boolean (value);
-      break;
-    case PROP_TOKEN_PARTS:
-      gst_vp8_enc->partitions = g_value_get_int (value);
-      break;
     default:
       break;
   }
@@ -592,18 +404,6 @@ gst_vp8_enc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_MODE:
       g_value_set_enum (value, gst_vp8_enc->mode);
       break;
-    case PROP_MINSECTION_PCT:
-      g_value_set_uint (value, gst_vp8_enc->minsection_pct);
-      break;
-    case PROP_MAXSECTION_PCT:
-      g_value_set_uint (value, gst_vp8_enc->maxsection_pct);
-      break;
-    case PROP_MIN_QUANTIZER:
-      g_value_set_int (value, gst_vp8_enc->min_quantizer);
-      break;
-    case PROP_MAX_QUANTIZER:
-      g_value_set_int (value, gst_vp8_enc->max_quantizer);
-      break;
     case PROP_QUALITY:
       g_value_set_double (value, gst_vp8_enc->quality);
       break;
@@ -631,35 +431,6 @@ gst_vp8_enc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_AUTO_ALT_REF_FRAMES:
       g_value_set_boolean (value, gst_vp8_enc->auto_alt_ref_frames);
       break;
-    case PROP_LAG_IN_FRAMES:
-      g_value_set_uint (value, gst_vp8_enc->lag_in_frames);
-      break;
-    case PROP_SHARPNESS:
-      g_value_set_int (value, gst_vp8_enc->sharpness);
-      break;
-    case PROP_NOISE_SENSITIVITY:
-      g_value_set_int (value, gst_vp8_enc->noise_sensitivity);
-      break;
-    case PROP_TUNE:
-#ifdef HAVE_VP8ENC_TUNING
-      g_value_set_enum (value, gst_vp8_enc->tuning);
-#else
-      GST_WARNING_OBJECT (gst_vp8_enc,
-          "The tuning property is unsupported by this libvpx");
-#endif
-      break;
-    case PROP_STATIC_THRESHOLD:
-      g_value_set_int (value, gst_vp8_enc->static_threshold);
-      break;
-    case PROP_DROP_FRAME:
-      g_value_set_int (value, gst_vp8_enc->drop_frame);
-      break;
-    case PROP_RESIZE_ALLOWED:
-      g_value_set_boolean (value, gst_vp8_enc->resize_allowed);
-      break;
-    case PROP_TOKEN_PARTS:
-      g_value_set_int (value, gst_vp8_enc->partitions);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -708,146 +479,193 @@ static gboolean
 gst_vp8_enc_set_format (GstBaseVideoEncoder * base_video_encoder,
     GstVideoState * state)
 {
-  GstVP8Enc *encoder;
-  vpx_codec_enc_cfg_t cfg;
-  vpx_codec_err_t status;
-  vpx_image_t *image;
-  guint8 *data = NULL;
-  GstCaps *caps;
-  gboolean ret;
-
-  encoder = GST_VP8_ENC (base_video_encoder);
   GST_DEBUG_OBJECT (base_video_encoder, "set_format");
 
-  if (encoder->inited) {
-    GST_DEBUG_OBJECT (base_video_encoder, "refusing renegotiation");
-    return FALSE;
+  return TRUE;
+}
+
+static GstCaps *
+gst_vp8_enc_get_caps (GstBaseVideoEncoder * base_video_encoder)
+{
+  GstCaps *caps;
+  const GstVideoState *state;
+  GstTagList *tags = NULL;
+  const GstTagList *iface_tags;
+  GstBuffer *stream_hdr, *vorbiscomment;
+  guint8 *data;
+  GstStructure *s;
+  GValue array = { 0 };
+  GValue value = { 0 };
+
+  state = gst_base_video_encoder_get_state (base_video_encoder);
+
+  caps = gst_caps_new_simple ("video/x-vp8",
+      "width", G_TYPE_INT, state->width,
+      "height", G_TYPE_INT, state->height,
+      "framerate", GST_TYPE_FRACTION, state->fps_n,
+      state->fps_d,
+      "pixel-aspect-ratio", GST_TYPE_FRACTION, state->par_n,
+      state->par_d, NULL);
+
+  s = gst_caps_get_structure (caps, 0);
+
+  /* put buffers in a fixed list */
+  g_value_init (&array, GST_TYPE_ARRAY);
+  g_value_init (&value, GST_TYPE_BUFFER);
+
+  /* Create Ogg stream-info */
+  stream_hdr = gst_buffer_new_and_alloc (26);
+  data = GST_BUFFER_DATA (stream_hdr);
+
+  GST_WRITE_UINT8 (data, 0x4F);
+  GST_WRITE_UINT32_BE (data + 1, 0x56503830);   /* "VP80" */
+  GST_WRITE_UINT8 (data + 5, 0x01);     /* stream info header */
+  GST_WRITE_UINT8 (data + 6, 1);        /* Major version 1 */
+  GST_WRITE_UINT8 (data + 7, 0);        /* Minor version 0 */
+  GST_WRITE_UINT16_BE (data + 8, state->width);
+  GST_WRITE_UINT16_BE (data + 10, state->height);
+  GST_WRITE_UINT24_BE (data + 12, state->par_n);
+  GST_WRITE_UINT24_BE (data + 15, state->par_d);
+  GST_WRITE_UINT32_BE (data + 18, state->fps_n);
+  GST_WRITE_UINT32_BE (data + 22, state->fps_d);
+
+  GST_BUFFER_FLAG_SET (stream_hdr, GST_BUFFER_FLAG_IN_CAPS);
+  gst_value_set_buffer (&value, stream_hdr);
+  gst_value_array_append_value (&array, &value);
+  g_value_unset (&value);
+  gst_buffer_unref (stream_hdr);
+
+  iface_tags =
+      gst_tag_setter_get_tag_list (GST_TAG_SETTER (base_video_encoder));
+  if (iface_tags) {
+    vorbiscomment =
+        gst_tag_list_to_vorbiscomment_buffer ((iface_tags) ? iface_tags : tags,
+        (const guint8 *) "OVP80\2 ", 7,
+        "Encoded with GStreamer vp8enc " PACKAGE_VERSION);
+
+    GST_BUFFER_FLAG_SET (vorbiscomment, GST_BUFFER_FLAG_IN_CAPS);
+
+    g_value_init (&value, GST_TYPE_BUFFER);
+    gst_value_set_buffer (&value, vorbiscomment);
+    gst_value_array_append_value (&array, &value);
+    g_value_unset (&value);
+    gst_buffer_unref (vorbiscomment);
   }
 
-  status = vpx_codec_enc_config_default (&vpx_codec_vp8_cx_algo, &cfg, 0);
-  if (status != VPX_CODEC_OK) {
-    GST_ELEMENT_ERROR (encoder, LIBRARY, INIT,
-        ("Failed to get default encoder configuration"), ("%s",
-            gst_vpx_error_name (status)));
-    return FALSE;
-  }
+  gst_structure_set_value (s, "streamheader", &array);
+  g_value_unset (&array);
 
-  cfg.g_w = state->width;
-  cfg.g_h = state->height;
-  cfg.g_timebase.num = state->fps_d;
-  cfg.g_timebase.den = state->fps_n;
+  return caps;
+}
 
-  cfg.g_error_resilient = encoder->error_resilient;
-  cfg.g_lag_in_frames = encoder->max_latency;
-  cfg.g_threads = encoder->threads;
-  cfg.rc_end_usage = encoder->mode;
-  cfg.rc_2pass_vbr_minsection_pct = encoder->minsection_pct;
-  cfg.rc_2pass_vbr_maxsection_pct = encoder->maxsection_pct;
-  /* Standalone qp-min do not make any sence, with bitrate=0 and qp-min=1
-   * encoder will use only default qp-max=63. Also this will make
-   * worst possbile quality.
-   */
-  if (encoder->bitrate != DEFAULT_BITRATE ||
-      encoder->max_quantizer != DEFAULT_MAX_QUANTIZER) {
-    cfg.rc_target_bitrate = encoder->bitrate / 1000;
-    cfg.rc_min_quantizer = encoder->min_quantizer;
-    cfg.rc_max_quantizer = encoder->max_quantizer;
-  } else {
-    cfg.rc_min_quantizer = (gint) (63 - encoder->quality * 6.2);
-    cfg.rc_max_quantizer = (gint) (63 - encoder->quality * 6.2);
-    cfg.rc_target_bitrate = encoder->bitrate;
-  }
-  cfg.rc_dropframe_thresh = encoder->drop_frame;
-  cfg.rc_resize_allowed = encoder->resize_allowed;
+static gboolean
+gst_vp8_enc_finish (GstBaseVideoEncoder * base_video_encoder)
+{
+  GstVP8Enc *encoder;
+  GstVideoFrame *frame;
+  int flags = 0;
+  vpx_codec_err_t status;
+  vpx_codec_iter_t iter = NULL;
+  const vpx_codec_cx_pkt_t *pkt;
 
-  cfg.kf_mode = VPX_KF_AUTO;
-  cfg.kf_min_dist = 0;
-  cfg.kf_max_dist = encoder->max_keyframe_distance;
+  GST_DEBUG_OBJECT (base_video_encoder, "finish");
 
-  cfg.g_pass = encoder->multipass_mode;
-  if (encoder->multipass_mode == VPX_RC_FIRST_PASS) {
-    encoder->first_pass_cache_content = g_byte_array_sized_new (4096);
-  } else if (encoder->multipass_mode == VPX_RC_LAST_PASS) {
-    GError *err = NULL;
-
-    if (!encoder->multipass_cache_file) {
-      GST_ELEMENT_ERROR (encoder, RESOURCE, OPEN_READ,
-          ("No multipass cache file provided"), (NULL));
-      return FALSE;
-    }
-
-    if (!g_file_get_contents (encoder->multipass_cache_file,
-            (gchar **) & encoder->last_pass_cache_content.buf,
-            &encoder->last_pass_cache_content.sz, &err)) {
-      GST_ELEMENT_ERROR (encoder, RESOURCE, OPEN_READ,
-          ("Failed to read multipass cache file provided"), ("%s",
-              err->message));
-      g_error_free (err);
-      return FALSE;
-    }
-    cfg.rc_twopass_stats_in = encoder->last_pass_cache_content;
-  }
-
-  status = vpx_codec_enc_init (&encoder->encoder, &vpx_codec_vp8_cx_algo,
-      &cfg, 0);
-  if (status != VPX_CODEC_OK) {
-    GST_ELEMENT_ERROR (encoder, LIBRARY, INIT,
-        ("Failed to initialize encoder"), ("%s", gst_vpx_error_name (status)));
-    return FALSE;
-  }
-
-  /* FIXME move this to a set_speed() function */
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_CPUUSED,
-      (encoder->speed == 0) ? 0 : (encoder->speed - 1));
-  if (status != VPX_CODEC_OK) {
-    GST_WARNING_OBJECT (encoder, "Failed to set VP8E_SET_CPUUSED to 0: %s",
-        gst_vpx_error_name (status));
-  }
-
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_NOISE_SENSITIVITY,
-      encoder->noise_sensitivity);
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_SHARPNESS,
-      encoder->sharpness);
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_STATIC_THRESHOLD,
-      encoder->static_threshold);
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_TOKEN_PARTITIONS,
-      encoder->partitions);
-#if 0
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_ARNR_MAXFRAMES,
-      encoder->arnr_maxframes);
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_ARNR_STRENGTH,
-      encoder->arnr_strength);
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_ARNR_TYPE,
-      encoder->arnr_type);
-#endif
-#ifdef HAVE_VP8ENC_TUNING
-  status = vpx_codec_control (&encoder->encoder, VP8E_SET_TUNING,
-      encoder->tuning);
-#endif
+  encoder = GST_VP8_ENC (base_video_encoder);
 
   status =
-      vpx_codec_control (&encoder->encoder, VP8E_SET_ENABLEAUTOALTREF,
-      (encoder->auto_alt_ref_frames ? 1 : 0));
-  if (status != VPX_CODEC_OK) {
-    GST_WARNING_OBJECT (encoder,
-        "Failed to set VP8E_ENABLEAUTOALTREF to %d: %s",
-        (encoder->auto_alt_ref_frames ? 1 : 0), gst_vpx_error_name (status));
+      vpx_codec_encode (&encoder->encoder, NULL, encoder->n_frames, 1, flags,
+      0);
+  if (status != 0) {
+    GST_ERROR_OBJECT (encoder, "encode returned %d %s", status,
+        gst_vpx_error_name (status));
+    return FALSE;
   }
 
-  cfg.g_lag_in_frames = encoder->lag_in_frames;
+  pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
+  while (pkt != NULL) {
+    GstBuffer *buffer;
+    GstVP8EncCoderHook *hook;
+    gboolean invisible, keyframe;
 
-  gst_base_video_encoder_set_latency (base_video_encoder, 0,
-      gst_util_uint64_scale (encoder->max_latency,
-          state->fps_d * GST_SECOND, state->fps_n));
-  encoder->inited = TRUE;
+    GST_DEBUG_OBJECT (encoder, "packet %u type %d", (guint) pkt->data.frame.sz,
+        pkt->kind);
 
-  /* prepare cached image buffer setup */
-  image = &encoder->image;
-  memset (image, 0, sizeof (image));
+    if (pkt->kind == VPX_CODEC_STATS_PKT
+        && encoder->multipass_mode == VPX_RC_FIRST_PASS) {
+      GST_LOG_OBJECT (encoder, "handling STATS packet");
+
+      g_byte_array_append (encoder->first_pass_cache_content,
+          pkt->data.twopass_stats.buf, pkt->data.twopass_stats.sz);
+
+      frame = gst_base_video_encoder_get_oldest_frame (base_video_encoder);
+      if (frame != NULL) {
+        buffer = gst_buffer_new ();
+        GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_PREROLL);
+        frame->src_buffer = buffer;
+        gst_base_video_encoder_finish_frame (base_video_encoder, frame);
+      }
+
+      pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
+      continue;
+    } else if (pkt->kind != VPX_CODEC_CX_FRAME_PKT) {
+      GST_LOG_OBJECT (encoder, "non frame pkt: %d", pkt->kind);
+      pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
+      continue;
+    }
+
+    invisible = (pkt->data.frame.flags & VPX_FRAME_IS_INVISIBLE) != 0;
+    keyframe = (pkt->data.frame.flags & VPX_FRAME_IS_KEY) != 0;
+    frame = gst_base_video_encoder_get_oldest_frame (base_video_encoder);
+    g_assert (frame != NULL);
+    hook = frame->coder_hook;
+
+    buffer = gst_buffer_new_and_alloc (pkt->data.frame.sz);
+
+    memcpy (GST_BUFFER_DATA (buffer), pkt->data.frame.buf, pkt->data.frame.sz);
+    frame->is_sync_point = frame->is_sync_point || keyframe;
+
+    if (hook->image)
+      g_slice_free (vpx_image_t, hook->image);
+    hook->image = NULL;
+
+    if (invisible) {
+      hook->invisible = g_list_append (hook->invisible, buffer);
+    } else {
+      frame->src_buffer = buffer;
+      gst_base_video_encoder_finish_frame (base_video_encoder, frame);
+      frame = NULL;
+    }
+
+    pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
+  }
+
+  if (encoder->multipass_mode == VPX_RC_FIRST_PASS
+      && encoder->multipass_cache_file) {
+    GError *err = NULL;
+
+    if (!g_file_set_contents (encoder->multipass_cache_file,
+            (const gchar *) encoder->first_pass_cache_content->data,
+            encoder->first_pass_cache_content->len, &err)) {
+      GST_ELEMENT_ERROR (encoder, RESOURCE, WRITE, (NULL),
+          ("Failed to write multipass cache file: %s", err->message));
+      g_error_free (err);
+    }
+  }
+
+  return TRUE;
+}
+
+static vpx_image_t *
+gst_vp8_enc_buffer_to_image (GstVP8Enc * enc, GstBuffer * buffer)
+{
+  vpx_image_t *image = g_slice_new0 (vpx_image_t);
+  guint8 *data = GST_BUFFER_DATA (buffer);
+  GstVideoState *state = &GST_BASE_VIDEO_CODEC (enc)->state;
 
   image->fmt = VPX_IMG_FMT_I420;
   image->bps = 12;
   image->x_chroma_shift = image->y_chroma_shift = 1;
+  image->img_data = data;
   image->w = image->d_w = state->width;
   image->h = image->d_h = state->height;
 
@@ -867,86 +685,145 @@ gst_vp8_enc_set_format (GstBaseVideoEncoder * base_video_encoder,
       data + gst_video_format_get_component_offset (state->format, 2,
       state->width, state->height);
 
-
-  caps = gst_caps_new_simple ("video/x-vp8",
-      "width", G_TYPE_INT, state->width,
-      "height", G_TYPE_INT, state->height,
-      "framerate", GST_TYPE_FRACTION, state->fps_n,
-      state->fps_d,
-      "pixel-aspect-ratio", GST_TYPE_FRACTION, state->par_n,
-      state->par_d, NULL);
-  {
-    GstStructure *s;
-    GstBuffer *stream_hdr, *vorbiscomment;
-    const GstTagList *iface_tags;
-    GValue array = { 0, };
-    GValue value = { 0, };
-    s = gst_caps_get_structure (caps, 0);
-
-    /* put buffers in a fixed list */
-    g_value_init (&array, GST_TYPE_ARRAY);
-    g_value_init (&value, GST_TYPE_BUFFER);
-
-    /* Create Ogg stream-info */
-    stream_hdr = gst_buffer_new_and_alloc (26);
-    data = GST_BUFFER_DATA (stream_hdr);
-
-    GST_WRITE_UINT8 (data, 0x4F);
-    GST_WRITE_UINT32_BE (data + 1, 0x56503830); /* "VP80" */
-    GST_WRITE_UINT8 (data + 5, 0x01);   /* stream info header */
-    GST_WRITE_UINT8 (data + 6, 1);      /* Major version 1 */
-    GST_WRITE_UINT8 (data + 7, 0);      /* Minor version 0 */
-    GST_WRITE_UINT16_BE (data + 8, state->width);
-    GST_WRITE_UINT16_BE (data + 10, state->height);
-    GST_WRITE_UINT24_BE (data + 12, state->par_n);
-    GST_WRITE_UINT24_BE (data + 15, state->par_d);
-    GST_WRITE_UINT32_BE (data + 18, state->fps_n);
-    GST_WRITE_UINT32_BE (data + 22, state->fps_d);
-
-    GST_BUFFER_FLAG_SET (stream_hdr, GST_BUFFER_FLAG_IN_CAPS);
-    gst_value_set_buffer (&value, stream_hdr);
-    gst_value_array_append_value (&array, &value);
-    g_value_unset (&value);
-    gst_buffer_unref (stream_hdr);
-
-    iface_tags =
-        gst_tag_setter_get_tag_list (GST_TAG_SETTER (base_video_encoder));
-    if (iface_tags) {
-      vorbiscomment =
-          gst_tag_list_to_vorbiscomment_buffer (iface_tags,
-          (const guint8 *) "OVP80\2 ", 7,
-          "Encoded with GStreamer vp8enc " PACKAGE_VERSION);
-
-      GST_BUFFER_FLAG_SET (vorbiscomment, GST_BUFFER_FLAG_IN_CAPS);
-
-      g_value_init (&value, GST_TYPE_BUFFER);
-      gst_value_set_buffer (&value, vorbiscomment);
-      gst_value_array_append_value (&array, &value);
-      g_value_unset (&value);
-      gst_buffer_unref (vorbiscomment);
-    }
-
-    gst_structure_set_value (s, "streamheader", &array);
-    g_value_unset (&array);
-  }
-
-  ret = gst_pad_set_caps (GST_BASE_VIDEO_CODEC_SRC_PAD (encoder), caps);
-  gst_caps_unref (caps);
-
-  return ret;
+  return image;
 }
 
-static GstFlowReturn
-gst_vp8_enc_process (GstVP8Enc * encoder)
+static const int speed_table[] = {
+  VPX_DL_BEST_QUALITY,
+  VPX_DL_GOOD_QUALITY,
+  VPX_DL_REALTIME,
+};
+
+static gboolean
+gst_vp8_enc_handle_frame (GstBaseVideoEncoder * base_video_encoder,
+    GstVideoFrame * frame)
 {
+  GstVP8Enc *encoder;
+  const GstVideoState *state;
+  vpx_codec_err_t status;
+  int flags = 0;
   vpx_codec_iter_t iter = NULL;
   const vpx_codec_cx_pkt_t *pkt;
-  GstBaseVideoEncoder *base_video_encoder;
+  vpx_image_t *image;
   GstVP8EncCoderHook *hook;
-  GstVideoFrame *frame;
-  GstFlowReturn ret = GST_FLOW_OK;
 
-  base_video_encoder = GST_BASE_VIDEO_ENCODER (encoder);
+  GST_DEBUG_OBJECT (base_video_encoder, "handle_frame");
+
+  encoder = GST_VP8_ENC (base_video_encoder);
+
+  state = gst_base_video_encoder_get_state (base_video_encoder);
+  encoder->n_frames++;
+
+  GST_DEBUG_OBJECT (base_video_encoder, "size %d %d", state->width,
+      state->height);
+
+  if (!encoder->inited) {
+    vpx_codec_enc_cfg_t cfg;
+
+    status = vpx_codec_enc_config_default (&vpx_codec_vp8_cx_algo, &cfg, 0);
+    if (status != VPX_CODEC_OK) {
+      GST_ELEMENT_ERROR (encoder, LIBRARY, INIT,
+          ("Failed to get default encoder configuration"), ("%s",
+              gst_vpx_error_name (status)));
+      return FALSE;
+    }
+
+    cfg.g_w = state->width;
+    cfg.g_h = state->height;
+    cfg.g_timebase.num = state->fps_d;
+    cfg.g_timebase.den = state->fps_n;
+
+    cfg.g_error_resilient = encoder->error_resilient;
+    cfg.g_lag_in_frames = encoder->max_latency;
+    cfg.g_threads = encoder->threads;
+    cfg.rc_end_usage = encoder->mode;
+    if (encoder->bitrate) {
+      cfg.rc_target_bitrate = encoder->bitrate / 1000;
+    } else {
+      cfg.rc_min_quantizer = 63 - encoder->quality * 5.0;
+      cfg.rc_max_quantizer = 63 - encoder->quality * 5.0;
+      cfg.rc_target_bitrate = encoder->bitrate;
+    }
+
+    cfg.kf_mode = VPX_KF_AUTO;
+    cfg.kf_min_dist = 0;
+    cfg.kf_max_dist = encoder->max_keyframe_distance;
+
+    cfg.g_pass = encoder->multipass_mode;
+    if (encoder->multipass_mode == VPX_RC_FIRST_PASS) {
+      encoder->first_pass_cache_content = g_byte_array_sized_new (4096);
+    } else if (encoder->multipass_mode == VPX_RC_LAST_PASS) {
+      GError *err = NULL;
+
+
+      if (!encoder->multipass_cache_file) {
+        GST_ELEMENT_ERROR (encoder, RESOURCE, OPEN_READ,
+            ("No multipass cache file provided"), (NULL));
+        return GST_FLOW_ERROR;
+      }
+
+      if (!g_file_get_contents (encoder->multipass_cache_file,
+              (gchar **) & encoder->last_pass_cache_content.buf,
+              &encoder->last_pass_cache_content.sz, &err)) {
+        GST_ELEMENT_ERROR (encoder, RESOURCE, OPEN_READ,
+            ("Failed to read multipass cache file provided"), ("%s",
+                err->message));
+        g_error_free (err);
+        return GST_FLOW_ERROR;
+      }
+      cfg.rc_twopass_stats_in = encoder->last_pass_cache_content;
+    }
+
+    status = vpx_codec_enc_init (&encoder->encoder, &vpx_codec_vp8_cx_algo,
+        &cfg, 0);
+    if (status != VPX_CODEC_OK) {
+      GST_ELEMENT_ERROR (encoder, LIBRARY, INIT,
+          ("Failed to initialize encoder"), ("%s",
+              gst_vpx_error_name (status)));
+      return GST_FLOW_ERROR;
+    }
+
+    status = vpx_codec_control (&encoder->encoder, VP8E_SET_CPUUSED, 0);
+    if (status != VPX_CODEC_OK) {
+      GST_WARNING_OBJECT (encoder, "Failed to set VP8E_SET_CPUUSED to 0: %s",
+          gst_vpx_error_name (status));
+    }
+
+    status =
+        vpx_codec_control (&encoder->encoder, VP8E_SET_ENABLEAUTOALTREF,
+        (encoder->auto_alt_ref_frames ? 1 : 0));
+    if (status != VPX_CODEC_OK) {
+      GST_WARNING_OBJECT (encoder,
+          "Failed to set VP8E_ENABLEAUTOALTREF to %d: %s",
+          (encoder->auto_alt_ref_frames ? 1 : 0), gst_vpx_error_name (status));
+    }
+
+    gst_base_video_encoder_set_latency (base_video_encoder, 0,
+        gst_util_uint64_scale (encoder->max_latency,
+            state->fps_d * GST_SECOND, state->fps_n));
+    encoder->inited = TRUE;
+  }
+
+  image = gst_vp8_enc_buffer_to_image (encoder, frame->sink_buffer);
+
+  hook = g_slice_new0 (GstVP8EncCoderHook);
+  hook->image = image;
+  frame->coder_hook = hook;
+
+  if (encoder->force_keyframe) {
+    flags |= VPX_EFLAG_FORCE_KF;
+  }
+
+  status = vpx_codec_encode (&encoder->encoder, image,
+      encoder->n_frames, 1, flags, speed_table[encoder->speed]);
+  if (status != 0) {
+    GST_ELEMENT_ERROR (encoder, LIBRARY, ENCODE,
+        ("Failed to encode frame"), ("%s", gst_vpx_error_name (status)));
+    g_slice_free (GstVP8EncCoderHook, hook);
+    frame->coder_hook = NULL;
+    g_slice_free (vpx_image_t, image);
+    return FALSE;
+  }
 
   pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
   while (pkt != NULL) {
@@ -997,118 +874,13 @@ gst_vp8_enc_process (GstVP8Enc * encoder)
       hook->invisible = g_list_append (hook->invisible, buffer);
     } else {
       frame->src_buffer = buffer;
-      ret = gst_base_video_encoder_finish_frame (base_video_encoder, frame);
+      gst_base_video_encoder_finish_frame (base_video_encoder, frame);
     }
 
     pkt = vpx_codec_get_cx_data (&encoder->encoder, &iter);
   }
 
-  return ret;
-}
-
-static GstFlowReturn
-gst_vp8_enc_finish (GstBaseVideoEncoder * base_video_encoder)
-{
-  GstVP8Enc *encoder;
-  int flags = 0;
-  vpx_codec_err_t status;
-
-  GST_DEBUG_OBJECT (base_video_encoder, "finish");
-
-  encoder = GST_VP8_ENC (base_video_encoder);
-
-  status =
-      vpx_codec_encode (&encoder->encoder, NULL, encoder->n_frames, 1, flags,
-      0);
-  if (status != 0) {
-    GST_ERROR_OBJECT (encoder, "encode returned %d %s", status,
-        gst_vpx_error_name (status));
-    return GST_FLOW_ERROR;
-  }
-
-  /* dispatch remaining frames */
-  gst_vp8_enc_process (encoder);
-
-  if (encoder->multipass_mode == VPX_RC_FIRST_PASS
-      && encoder->multipass_cache_file) {
-    GError *err = NULL;
-
-    if (!g_file_set_contents (encoder->multipass_cache_file,
-            (const gchar *) encoder->first_pass_cache_content->data,
-            encoder->first_pass_cache_content->len, &err)) {
-      GST_ELEMENT_ERROR (encoder, RESOURCE, WRITE, (NULL),
-          ("Failed to write multipass cache file: %s", err->message));
-      g_error_free (err);
-    }
-  }
-
-  return GST_FLOW_OK;
-}
-
-static vpx_image_t *
-gst_vp8_enc_buffer_to_image (GstVP8Enc * enc, GstBuffer * buffer)
-{
-  vpx_image_t *image = g_slice_new (vpx_image_t);
-  guint8 *data = GST_BUFFER_DATA (buffer);
-
-  memcpy (image, &enc->image, sizeof (*image));
-
-  image->img_data = data;
-  image->planes[VPX_PLANE_Y] += (data - (guint8 *) NULL);
-  image->planes[VPX_PLANE_U] += (data - (guint8 *) NULL);
-  image->planes[VPX_PLANE_V] += (data - (guint8 *) NULL);
-
-  return image;
-}
-
-static GstFlowReturn
-gst_vp8_enc_handle_frame (GstBaseVideoEncoder * base_video_encoder,
-    GstVideoFrame * frame)
-{
-  GstVP8Enc *encoder;
-  const GstVideoState *state;
-  vpx_codec_err_t status;
-  int flags = 0;
-  vpx_image_t *image;
-  GstVP8EncCoderHook *hook;
-  int quality;
-
-  GST_DEBUG_OBJECT (base_video_encoder, "handle_frame");
-
-  encoder = GST_VP8_ENC (base_video_encoder);
-
-  state = gst_base_video_encoder_get_state (base_video_encoder);
-  encoder->n_frames++;
-
-  GST_DEBUG_OBJECT (base_video_encoder, "size %d %d", state->width,
-      state->height);
-
-  image = gst_vp8_enc_buffer_to_image (encoder, frame->sink_buffer);
-
-  hook = g_slice_new0 (GstVP8EncCoderHook);
-  hook->image = image;
-  frame->coder_hook = hook;
-  frame->coder_hook_destroy_notify =
-      (GDestroyNotify) gst_vp8_enc_coder_hook_free;
-
-  if (frame->force_keyframe) {
-    flags |= VPX_EFLAG_FORCE_KF;
-  }
-
-  quality = (encoder->speed == 0) ? VPX_DL_BEST_QUALITY : VPX_DL_GOOD_QUALITY;
-
-  status = vpx_codec_encode (&encoder->encoder, image,
-      encoder->n_frames, 1, flags, quality);
-  if (status != 0) {
-    GST_ELEMENT_ERROR (encoder, LIBRARY, ENCODE,
-        ("Failed to encode frame"), ("%s", gst_vpx_error_name (status)));
-    g_slice_free (GstVP8EncCoderHook, hook);
-    frame->coder_hook = NULL;
-    g_slice_free (vpx_image_t, image);
-    return FALSE;
-  }
-
-  return gst_vp8_enc_process (encoder);
+  return TRUE;
 }
 
 static guint64
@@ -1123,6 +895,13 @@ _to_granulepos (guint64 frame_end_number, guint inv_count, guint keyframe_dist)
   return granulepos;
 }
 
+static void
+_gst_mini_object_unref0 (GstMiniObject * obj)
+{
+  if (obj)
+    gst_mini_object_unref (obj);
+}
+
 static GstFlowReturn
 gst_vp8_enc_shape_output (GstBaseVideoEncoder * base_video_encoder,
     GstVideoFrame * frame)
@@ -1155,8 +934,6 @@ gst_vp8_enc_shape_output (GstBaseVideoEncoder * base_video_encoder,
       encoder->keyframe_distance++;
     }
 
-    GST_BUFFER_TIMESTAMP (buf) = GST_BUFFER_TIMESTAMP (frame->src_buffer);
-    GST_BUFFER_DURATION (buf) = 0;
     GST_BUFFER_OFFSET_END (buf) =
         _to_granulepos (frame->presentation_frame_number + 1,
         inv_count, encoder->keyframe_distance);
@@ -1164,8 +941,7 @@ gst_vp8_enc_shape_output (GstBaseVideoEncoder * base_video_encoder,
         gst_util_uint64_scale (frame->presentation_frame_number + 1,
         GST_SECOND * state->fps_d, state->fps_n);
 
-    gst_buffer_set_caps (buf,
-        GST_PAD_CAPS (GST_BASE_VIDEO_CODEC_SRC_PAD (base_video_encoder)));
+    gst_buffer_set_caps (buf, GST_BASE_VIDEO_CODEC (base_video_encoder)->caps);
     ret = gst_pad_push (GST_BASE_VIDEO_CODEC_SRC_PAD (base_video_encoder), buf);
 
     if (ret != GST_FLOW_OK) {
@@ -1198,13 +974,21 @@ gst_vp8_enc_shape_output (GstBaseVideoEncoder * base_video_encoder,
   }
 
 done:
+  if (hook) {
+    g_list_foreach (hook->invisible, (GFunc) _gst_mini_object_unref0, NULL);
+    g_list_free (hook->invisible);
+    g_slice_free (GstVP8EncCoderHook, hook);
+    frame->coder_hook = NULL;
+  }
+
   return ret;
 }
 
 static gboolean
-gst_vp8_enc_sink_event (GstBaseVideoEncoder * benc, GstEvent * event)
+gst_vp8_enc_sink_event (GstPad * pad, GstEvent * event)
 {
-  GstVP8Enc *enc = GST_VP8_ENC (benc);
+  GstVP8Enc *enc = GST_VP8_ENC (gst_pad_get_parent (pad));
+  gboolean ret;
 
   if (GST_EVENT_TYPE (event) == GST_EVENT_TAG) {
     GstTagList *list;
@@ -1215,8 +999,10 @@ gst_vp8_enc_sink_event (GstBaseVideoEncoder * benc, GstEvent * event)
     gst_tag_setter_merge_tags (setter, list, mode);
   }
 
-  /* just peeked, baseclass handles the rest */
-  return FALSE;
+  ret = enc->base_sink_event_func (pad, event);
+  gst_object_unref (enc);
+
+  return ret;
 }
 
 #endif /* HAVE_VP8_ENCODER */
