@@ -57,7 +57,6 @@
 
 #include "aiffparse.h"
 #include <gst/audio/audio.h>
-#include <gst/tag/tag.h>
 #include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY (aiffparse_debug);
@@ -105,10 +104,10 @@ gst_aiff_parse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &src_template_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template_factory));
 
   gst_element_class_set_details_simple (element_class,
       "AIFF audio demuxer", "Codec/Demuxer/Audio",
@@ -164,11 +163,6 @@ gst_aiff_parse_reset (GstAiffParse * aiff)
     gst_adapter_clear (aiff->adapter);
     aiff->adapter = NULL;
   }
-
-  if (aiff->tags != NULL) {
-    gst_tag_list_free (aiff->tags);
-    aiff->tags = NULL;
-  }
 }
 
 static void
@@ -898,38 +892,6 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
         }
         break;
       }
-      case GST_MAKE_FOURCC ('I', 'D', '3', ' '):{
-        GstTagList *tags;
-
-        if (aiff->streaming) {
-          if (!gst_aiff_parse_peek_chunk (aiff, &tag, &size))
-            return GST_FLOW_OK;
-
-          gst_adapter_flush (aiff->adapter, 8);
-          aiff->offset += 8;
-
-          buf = gst_adapter_take_buffer (aiff->adapter, size);
-        } else {
-          if ((res = gst_aiff_parse_read_chunk (aiff,
-                      &aiff->offset, &tag, &buf)) != GST_FLOW_OK)
-            return res;
-        }
-
-        GST_LOG_OBJECT (aiff, "ID3 chunk of size %u", GST_BUFFER_SIZE (buf));
-
-        tags = gst_tag_list_from_id3v2_tag (buf);
-        gst_buffer_unref (buf);
-
-        GST_INFO_OBJECT (aiff, "ID3 tags: %" GST_PTR_FORMAT, tags);
-
-        if (aiff->tags == NULL) {
-          aiff->tags = tags;
-        } else {
-          gst_tag_list_insert (aiff->tags, tags, GST_TAG_MERGE_APPEND);
-          gst_tag_list_free (tags);
-        }
-        break;
-      }
       default:
         gst_aiff_parse_ignore_chunk (aiff, buf, tag, size);
     }
@@ -1155,11 +1117,6 @@ iterate_adapter:
     gst_pad_push_event (aiff->srcpad, aiff->start_segment);
     aiff->start_segment = NULL;
   }
-  if (G_UNLIKELY (aiff->tags != NULL)) {
-    gst_element_found_tags_for_pad (GST_ELEMENT_CAST (aiff), aiff->srcpad,
-        aiff->tags);
-    aiff->tags = NULL;
-  }
 
   obtained = GST_BUFFER_SIZE (buf);
 
