@@ -156,7 +156,9 @@ static void id3v2_frame_finish (GstId3v2Tag * tag, GstId3v2Frame * frame);
 static guint id3v2_frame_get_size (GstId3v2Tag * tag, GstId3v2Frame * frame);
 
 static void id3v2_tag_add_text_frame (GstId3v2Tag * tag,
-    const gchar * frame_id, gchar ** strings, int num_strings);
+    const gchar * frame_id, const gchar ** strings, int num_strings);
+static void id3v2_tag_add_simple_text_frame (GstId3v2Tag * tag,
+    const gchar * frame_id, const gchar * string);
 
 static gboolean
 id3v2_tag_init (GstId3v2Tag * tag, guint major_version)
@@ -374,10 +376,11 @@ id3v2_frame_write_string (GstId3v2Frame * frame, int encoding,
 
     /* Write the BOM */
     id3v2_frame_write_bytes (frame, (const guint8 *) bom, 2);
-    /* NUL terminator is 2 bytes, if present */
-    terminator_length = null_terminate ? 2 : 0;
-    id3v2_frame_write_bytes (frame, (const guint8 *) utf16,
-        utf16len + terminator_length);
+    id3v2_frame_write_bytes (frame, (const guint8 *) utf16, utf16len);
+    if (null_terminate) {
+      /* NUL terminator is 2 bytes, if present. */
+      id3v2_frame_write_uint16 (frame, 0);
+    }
 
     g_free (utf16);
   } else {
@@ -390,7 +393,7 @@ id3v2_frame_write_string (GstId3v2Frame * frame, int encoding,
 
 static void
 id3v2_tag_add_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
-    gchar ** strings_utf8, int num_strings)
+    const gchar ** strings_utf8, int num_strings)
 {
   GstId3v2Frame frame;
   guint len, i;
@@ -433,20 +436,27 @@ id3v2_tag_add_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
   g_array_append_val (tag->frames, frame);
 }
 
+static void
+id3v2_tag_add_simple_text_frame (GstId3v2Tag * tag, const gchar * frame_id,
+    const gchar * string)
+{
+  id3v2_tag_add_text_frame (tag, frame_id, (const gchar **) &string, 1);
+}
+
 /* ====================================================================== */
 
 static void
 add_text_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * frame_id)
 {
-  gchar **strings;
+  const gchar **strings;
   guint n, i;
 
   GST_LOG ("Adding '%s' frame", frame_id);
 
-  strings = g_new0 (gchar *, num_tags + 1);
+  strings = g_new0 (const gchar *, num_tags + 1);
   for (n = 0, i = 0; n < num_tags; ++n) {
-    if (gst_tag_list_get_string_index (list, tag, n, &strings[i]) &&
+    if (gst_tag_list_peek_string_index (list, tag, n, &strings[i]) &&
         strings[i] != NULL) {
       GST_LOG ("%s: %s[%u] = '%s'", frame_id, tag, i, strings[i]);
       ++i;
@@ -459,7 +469,7 @@ add_text_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
 
-  g_strfreev (strings);
+  g_free (strings);
 }
 
 static void
@@ -566,7 +576,7 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
       GST_DEBUG ("Setting %s to %s (frame_id = %s)", tag, tag_str, frame_id);
 
-      id3v2_tag_add_text_frame (id3v2tag, frame_id, &tag_str, 1);
+      id3v2_tag_add_simple_text_frame (id3v2tag, frame_id, tag_str);
       g_free (tag_str);
     }
   } else if (corr[idx].corr_count == NULL) {
@@ -579,7 +589,7 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
       gchar *tag_str = g_strdup_printf ("0/%u", count);
       GST_DEBUG ("Setting %s to %s (frame_id = %s)", tag, tag_str, frame_id);
 
-      id3v2_tag_add_text_frame (id3v2tag, frame_id, &tag_str, 1);
+      id3v2_tag_add_simple_text_frame (id3v2tag, frame_id, tag_str);
       g_free (tag_str);
     }
   }
@@ -589,6 +599,31 @@ add_count_or_num_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   }
 }
 
+static void
+add_bpm_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
+    const gchar * tag, guint num_tags, const gchar * unused)
+{
+  gdouble bpm;
+
+  GST_LOG ("Adding BPM frame");
+
+  if (gst_tag_list_get_double (list, tag, &bpm)) {
+    gchar *tag_str;
+
+    /* bpm is stored as an integer in id3 tags, but is a double in
+     * tag lists.
+     */
+    tag_str = g_strdup_printf ("%u", (guint) bpm);
+    GST_DEBUG ("Setting %s to %s", tag, tag_str);
+    id3v2_tag_add_simple_text_frame (id3v2tag, "TBPM", tag_str);
+    g_free (tag_str);
+  }
+
+  if (num_tags > 1) {
+    GST_WARNING ("more than one %s, can only handle one", tag);
+  }
+}
+
 static void
 add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * unused)
@@ -597,9 +632,9 @@ add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
   GST_LOG ("Adding comment frames");
   for (n = 0; n < num_tags; ++n) {
-    gchar *s = NULL;
+    const gchar *s = NULL;
 
-    if (gst_tag_list_get_string_index (list, tag, n, &s) && s != NULL) {
+    if (gst_tag_list_peek_string_index (list, tag, n, &s) && s != NULL) {
       gchar *desc = NULL, *val = NULL, *lang = NULL;
       int desclen, vallen, encoding1, encoding2, encoding;
       GstId3v2Frame frame;
@@ -642,7 +677,6 @@ add_comment_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
       g_array_append_val (id3v2tag->frames, frame);
     }
-    g_free (s);
   }
 }
 
@@ -743,9 +777,9 @@ add_musicbrainz_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   g_assert (idx < G_N_ELEMENTS (mb_ids));
 
   for (i = 0; i < num_tags; ++i) {
-    gchar *id_str;
+    const gchar *id_str;
 
-    if (gst_tag_list_get_string_index (list, tag, 0, &id_str) && id_str) {
+    if (gst_tag_list_peek_string_index (list, tag, 0, &id_str) && id_str) {
       /* add two frames, one with the ID the musicbrainz.org spec mentions
        * and one with the ID that applications use in the real world */
       GstId3v2Frame frame1, frame2;
@@ -766,8 +800,6 @@ add_musicbrainz_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
           mb_ids[idx].realworld_id, TRUE);
       id3v2_frame_write_string (&frame2, encoding, id_str, FALSE);
       g_array_append_val (id3v2tag->frames, frame2);
-
-      g_free (id_str);
     }
   }
 }
@@ -777,9 +809,9 @@ add_unique_file_id_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * unused)
 {
   const gchar *origin = "http://musicbrainz.org";
-  gchar *id_str = NULL;
+  const gchar *id_str = NULL;
 
-  if (gst_tag_list_get_string_index (list, tag, 0, &id_str) && id_str) {
+  if (gst_tag_list_peek_string_index (list, tag, 0, &id_str) && id_str) {
     GstId3v2Frame frame;
 
     GST_LOG ("Adding %s (%s): %s", tag, origin, id_str);
@@ -790,8 +822,6 @@ add_unique_file_id_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     id3v2_frame_write_bytes (&frame, (const guint8 *) id_str,
         strlen (id_str) + 1);
     g_array_append_val (id3v2tag->frames, frame);
-
-    g_free (id_str);
   }
 }
 
@@ -834,7 +864,7 @@ add_date_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   }
 
   if (strings[0] != NULL) {
-    id3v2_tag_add_text_frame (id3v2tag, frame_id, strings, i);
+    id3v2_tag_add_text_frame (id3v2tag, frame_id, (const gchar **) strings, i);
   } else {
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
@@ -856,9 +886,9 @@ add_encoder_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
 
   strings = g_new0 (gchar *, num_tags + 1);
   for (n = 0; n < num_tags; ++n) {
-    gchar *encoder = NULL;
+    const gchar *encoder = NULL;
 
-    if (gst_tag_list_get_string_index (list, tag, n, &encoder) && encoder) {
+    if (gst_tag_list_peek_string_index (list, tag, n, &encoder) && encoder) {
       guint encoder_version;
       gchar *s;
 
@@ -872,12 +902,11 @@ add_encoder_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
       GST_LOG ("encoder[%u] = '%s'", n, s);
       strings[i] = s;
       i++;
-      g_free (encoder);
     }
   }
 
   if (strings[0] != NULL) {
-    id3v2_tag_add_text_frame (id3v2tag, "TSSE", strings, i);
+    id3v2_tag_add_text_frame (id3v2tag, "TSSE", (const gchar **) strings, i);
   } else {
     GST_WARNING ("Empty list for tag %s, skipping", tag);
   }
@@ -889,12 +918,12 @@ static void
 add_uri_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     const gchar * tag, guint num_tags, const gchar * frame_id)
 {
-  gchar *url = NULL;
+  const gchar *url = NULL;
 
   g_assert (frame_id != NULL);
 
   /* URI tags are limited to one of each per taglist */
-  if (gst_tag_list_get_string_index (list, tag, 0, &url) && url != NULL) {
+  if (gst_tag_list_peek_string_index (list, tag, 0, &url) && url != NULL) {
     guint url_len;
 
     url_len = strlen (url);
@@ -907,8 +936,6 @@ add_uri_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
     } else {
       GST_WARNING ("Tag %s does not contain a valid URI (%s)", tag, url);
     }
-
-    g_free (url);
   }
 }
 
@@ -925,7 +952,7 @@ add_relative_volume_tag (GstId3v2Tag * id3v2tag, const GstTagList * list,
   gint16 gain_int;
   guint8 peak_bits;
   GstId3v2Frame frame;
-  gchar *frame_id;
+  const gchar *frame_id;
 
   /* figure out tag names and the identification string to use */
   if (strcmp (tag, GST_TAG_TRACK_PEAK) == 0 ||
@@ -1014,11 +1041,13 @@ static const struct
   {
     /* Simple text tags */
   GST_TAG_ARTIST, add_text_tag, "TPE1"}, {
+  GST_TAG_ALBUM_ARTIST, add_text_tag, "TPE2"}, {
   GST_TAG_TITLE, add_text_tag, "TIT2"}, {
   GST_TAG_ALBUM, add_text_tag, "TALB"}, {
   GST_TAG_COPYRIGHT, add_text_tag, "TCOP"}, {
   GST_TAG_COMPOSER, add_text_tag, "TCOM"}, {
   GST_TAG_GENRE, add_text_tag, "TCON"}, {
+  GST_TAG_ENCODED_BY, add_text_tag, "TENC"}, {
 
     /* Private frames */
   GST_ID3_DEMUX_TAG_ID3V2_FRAME, add_id3v2frame_tag, NULL}, {
@@ -1033,6 +1062,9 @@ static const struct
   GST_TAG_COMMENT, add_comment_tag, NULL}, {
   GST_TAG_EXTENDED_COMMENT, add_comment_tag, NULL}, {
 
+    /* BPM tag */
+  GST_TAG_BEATS_PER_MINUTE, add_bpm_tag, NULL}, {
+
     /* Images */
   GST_TAG_IMAGE, add_image_tag, NULL}, {
   GST_TAG_PREVIEW_IMAGE, add_image_tag, NULL}, {
@@ -1153,7 +1185,8 @@ latin1_convert (const GstTagList * list, const gchar * tag,
 
   /* Convert to Latin-1 (ISO-8859-1), replacing unrepresentable characters
      with '?' */
-  latin1 = g_convert_with_fallback (str, -1, "ISO-8859-1", "UTF-8", "?",
+  latin1 =
+      g_convert_with_fallback (str, -1, "ISO-8859-1", "UTF-8", (char *) "?",
       NULL, &len, NULL);
 
   if (latin1 != NULL && *latin1 != '\0') {
@@ -1193,12 +1226,12 @@ static void
 genre_v1_convert (const GstTagList * list, const gchar * tag,
     guint8 * dst, int maxlen, gboolean * wrote_tag)
 {
-  gchar *str;
+  const gchar *str;
   int genreidx = -1;
   guint i, max;
 
   /* We only support one genre */
-  if (!gst_tag_list_get_string_index (list, tag, 0, &str) || str == NULL)
+  if (!gst_tag_list_peek_string_index (list, tag, 0, &str) || str == NULL)
     return;
 
   max = gst_tag_id3_genre_count ();
@@ -1215,8 +1248,6 @@ genre_v1_convert (const GstTagList * list, const gchar * tag,
     *dst = (guint8) genreidx;
     *wrote_tag = TRUE;
   }
-
-  g_free (str);
 }
 
 static void
