@@ -624,8 +624,8 @@ gst_vdp_mpeg_dec_base_init (gpointer gclass)
       "Decode mpeg stream with vdpau",
       "Carl-Anton Ingmarsson <ca.ingmarsson@gmail.com>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
 }
 
 /* initialize the vdpaumpegdecoder's class */
